from formencode.validators import FancyValidator
from formencode import Invalid
import urllib, urllib2

import gettext
_ = gettext.gettext

class ReCaptchaValidator(FancyValidator):
    """
    @see formencode.validators.FieldsMatch
    """

    messages = {
        'incorrect': _("Incorrect value."),
        'missing':  _("Missing value."),
    }

    verify_server           = "api-verify.recaptcha.net"
    __unpackargs__ = ('*', 'field_names')

    validate_partial_form   = True
    validate_partial_python = None
    validate_partial_other = None

    def __init__(self, private_key, remote_ip, *args, **kw):
        super(ReCaptchaValidator, self).__init__(args, kw)
        self.private_key = private_key
        self.remote_ip = remote_ip
        self.field_names = ['recaptcha_challenge_field',
                            'recaptcha_response_field']
    
    def validate_partial(self, field_dict, state):
        for name in self.field_names:
            if not field_dict.has_key(name):
                return
        self.validate_python(field_dict, state)

    def validate_python(self, field_dict, state):
        challenge = field_dict['recaptcha_challenge_field']
        response = field_dict['recaptcha_response_field']
        if response == '' or challenge == '':
            error = Invalid(self.message('missing', state), field_dict, state)
            error.error_dict = {'recaptcha_response_field':'Missing value'}
            raise error
        params = urllib.urlencode({
            'privatekey': self.private_key,
            'remoteip' : self.remote_ip,
            'challenge': challenge,
            'response' : response,
            })
        request = urllib2.Request(
            url = "http://%s/verify" % self.verify_server,
            data = params,
            headers = {"Content-type": "application/x-www-form-urlencoded",
                       "User-agent": "reCAPTCHA Python"
                      }
            )
        
        httpresp = urllib2.urlopen(request)
        return_values = httpresp.read().splitlines();
        httpresp.close();
        return_code = return_values[0]
        if not return_code == "true":
            error = Invalid(self.message('incorrect', state), field_dict, state)
            error.error_dict = {'recaptcha_response_field':self.message('incorrect', state)}
            raise error
        return True



# Here you can create samples of your widgets by providing default parameters,
# inserting them in a container widget, mixing them with other widgets, etc...
# These samples will appear in the (unimplemented yet) widget browser.



from tw.api import Widget, JSLink, CSSLink
from genshi import HTML
#__all__ = ["Twrecaptcha"]

# declare your static resources here

## JS dependencies can be listed at 'javascript' so they'll get included
## before
# my_js = JSLink(modname=__name__, 
#                filename='static/twrecaptcha.js', javascript=[])

# my_css = CSSLink(modname=__name__, filename='static/twrecaptcha.css')

from recaptcha.client.captcha import *
from tw.forms import InputField, FormField, ContainerMixin

class ReCaptchaWidget(InputField):
    #template = """<div id="${id}">${captcha_response}</div>"""
    ## You can also define the template in a separate package and refer to it
    ## using Buffet style uris
    #template = "toscawidgets.widgets.twrecaptcha.templates.twrecaptcha"

    engine_name='genshi'
    #javascript = [my_js]
    #css = [my_css]
    params = ['captcha_response','public_key', 'server', 'use_ssl', 'error_param']

    template = """<div><script type="text/javascript" src="${server}/challenge?k=${public_key}${error_param}"></script>
<noscript>
  <iframe src="${server}/noscript?k=${public_key}${error_param}" height="300" width="500" frameborder="0"></iframe><br />
  <textarea name="recaptcha_challenge_field" rows="3" cols="40"></textarea>
  <input type='hidden' name='recaptcha_response_field' value='manual_challenge' />
</noscript></div>
""" 
    
    def __init__(self, id=None, public_key=None, use_ssl=False, error_param=None, parent=None, children=[], **kw):
        """Initialize the widget here. The widget's initial state shall be
        determined solely by the arguments passed to this function; two
        widgets initialized with the same args. should behave in *exactly* the
        same way. You should *not* rely on any external source to determine
        initial state."""
        self.public_key = public_key
        self.error_param = ''
        if error_param:
            self.error_param = '&error=%s'%error_param
        self.server = API_SERVER
        if use_ssl:
            self.server = API_SSL_SERVER
        super(ReCaptchaWidget, self).__init__('recaptcha_response_field', parent, children, **kw)

    def update_params(self, d):
        """This method is called every time the widget is displayed. It's task
        is to prepare all variables that are sent to the template. Those
        variables can accessed as attributes of d."""
        d['captcha_response'] = HTML(displayhtml(self.public_key))
        return super(ReCaptchaWidget, self).update_params(d)




# Metadata for the widget EGG. Should leave it for the (unimplemented) widget
# tracker to track it

__PROJECT__ = 'tw.recaptcha'
__DESCRIPTION__ = 'Handy ReCaptcha Toscawidget'
__URL__ = 'http://toscawidgets.org/'
__VERSION__ = '0.8'
__AUTHOR__ = 'Christopher Perkins'
__EMAIL__ = 'chris@percious.com'
__COPYRIGHT__ = "Copyright 2008 Christopher Perkins"
__LICENSE__ = 'MIT'
